dofile("Base.rte/Constants.lua")

function OneManArmy:StartActivity()
	-- Check if we already have a brain assigned
	for player = Activity.PLAYER_1, Activity.MAXPLAYERCOUNT - 1 do
		if self:PlayerActive(player) and self:PlayerHuman(player) then
			if not self:GetPlayerBrain(player) then
				local team = self:GetTeamOfPlayer(player);
				local foundBrain = MovableMan:GetUnassignedBrain(team);
				-- If we can't find an unassigned brain in the scene to give the player, then force to go into editing mode to place one
				if not foundBrain then
					foundBrain = CreateAHuman("Soldier Light", "Coalition.rte")
					local techName = self:GetTeamTech(team);
					local techModule = PresetMan:GetModuleID(techName);
					if techModule ~= -1 then
						foundBrain = RandomAHuman("Actors - Light", techName);
					end
					local playerGlove = "Boxing Glove";
					if self.Difficulty <= GameActivity.CAKEDIFFICULTY then
						self.enemyGlove = "Boxing Glove";
						playerGlove = "Super Glove";
					elseif self.Difficulty <= GameActivity.EASYDIFFICULTY then
						self.enemyGlove = "Boxing Glove";
						playerGlove = "Golden Glove";
					elseif self.Difficulty <= GameActivity.MEDIUMDIFFICULTY then
						self.enemyGlove = "Boxing Glove";
						playerGlove = "Golden Glove";
					elseif self.Difficulty <= GameActivity.HARDDIFFICULTY then
						self.enemyGlove = "Golden Glove";
						playerGlove = "Golden Glove";
					elseif self.Difficulty <= GameActivity.NUTSDIFFICULTY then
						self.enemyGlove = "Golden Glove";
						playerGlove = "Super Glove";
					elseif self.Difficulty <= GameActivity.MAXDIFFICULTY then
						self.enemyGlove = "Super Glove";
						playerGlove = "Super Glove";
					end
					for i = 1, 2 do
						foundBrain:AddInventoryItem(CreateHDFirearm(playerGlove));
					end
					-- reinforce arms
					foundBrain.FGArm.GibWoundLimit = 	foundBrain.FGArm.GibWoundLimit * 100;
					foundBrain.FGArm.JointStrength = 	foundBrain.FGArm.JointStrength * 100;
					
					foundBrain.BGArm.GibWoundLimit = 	foundBrain.BGArm.GibWoundLimit * 100;
					foundBrain.BGArm.JointStrength = 	foundBrain.BGArm.JointStrength * 100;

					foundBrain.Pos = SceneMan:MovePointToGround(Vector(math.random(0, SceneMan.SceneWidth), 0), 0, 0) + Vector(0, -50)
					foundBrain.Team = self:GetTeamOfPlayer(player)
					MovableMan:AddActor(foundBrain)
					-- Set the found brain to be the selected actor at start
					self:SetPlayerBrain(foundBrain, player)
					self:SwitchToActor(foundBrain, player, self:GetTeamOfPlayer(player))
					self:SetLandingZone(self:GetPlayerBrain(player).Pos, player)
					-- Set the observation target to the brain, so that if/when it dies, the view flies to it in observation mode
					self:SetObservationTarget(self:GetPlayerBrain(player).Pos, player)
				else
					-- Set the found brain to be the selected actor at start
					self:SetPlayerBrain(foundBrain, player)
					self:SwitchToActor(foundBrain, player, self:GetTeamOfPlayer(player))
					self:SetLandingZone(self:GetPlayerBrain(player).Pos, player)
					-- Set the observation target to the brain, so that if/when it dies, the view flies to it in observation mode
					self:SetObservationTarget(self:GetPlayerBrain(player).Pos, player)
				end
			end
		end
	end
	
	-- Select a tech for the CPU player
	self.CPUTechName = "Ronin.rte";
	self.CPUTechName = self:GetTeamTech(self.CPUTeam);
	self.ESpawnTimer = Timer();
	self.SurvivalTimer = Timer()
	
	if self.Difficulty <= GameActivity.CAKEDIFFICULTY then
		self:SetTeamAISkill(self.CPUTeam, 30)	-- set AI skill based on difficulty
		self.TimeLimit = 5*60000+5000
		self.timeDisplay = "five minutes"
		self.BaseSpawnTime = 6000
		self.RandomSpawnTime = 3000
	elseif self.Difficulty <= GameActivity.EASYDIFFICULTY then
		self:SetTeamAISkill(self.CPUTeam, 40)
		self.TimeLimit = 5*60000+5000
		self.timeDisplay = "five minutes"
		self.BaseSpawnTime = 6000
		self.RandomSpawnTime = 4000
	elseif self.Difficulty <= GameActivity.MEDIUMDIFFICULTY then
		self:SetTeamAISkill(self.CPUTeam, 50)
		self.TimeLimit = 5*60000+5000
		self.timeDisplay = "five minutes"
		self.BaseSpawnTime = 6000
		self.RandomSpawnTime = 5000
	elseif self.Difficulty <= GameActivity.HARDDIFFICULTY then
		self:SetTeamAISkill(self.CPUTeam, 60)
		self.TimeLimit = 5*60000+5000
		self.timeDisplay = "five minutes"
		self.BaseSpawnTime = 7000
		self.RandomSpawnTime = 4000
	elseif self.Difficulty <= GameActivity.NUTSDIFFICULTY then
		self:SetTeamAISkill(self.CPUTeam, 70)
		self.TimeLimit = 8*60000+5000
		self.timeDisplay = "eight minutes"
		self.BaseSpawnTime = 8000
		self.RandomSpawnTime = 4000
	elseif self.Difficulty <= GameActivity.MAXDIFFICULTY then
		self:SetTeamAISkill(self.CPUTeam, 80)
		self.TimeLimit = 10*60000+5000
		self.timeDisplay = "ten minutes"
		self.BaseSpawnTime = 6000
		self.RandomSpawnTime = 4000
	end

	self.StartTimer = Timer()
	ActivityMan:GetActivity():SetTeamFunds(0,Activity.TEAM_1)
	ActivityMan:GetActivity():SetTeamFunds(0,Activity.TEAM_2)
	ActivityMan:GetActivity():SetTeamFunds(0,Activity.TEAM_3)
	ActivityMan:GetActivity():SetTeamFunds(0,Activity.TEAM_4)
	
	-- CPU Funds are unlimited
	self:SetTeamFunds(1000000, self.CPUTeam);
	
	self.TimeLeft = 500
end

function OneManArmy:EndActivity()
	-- Play sad music if no humans are left
	if self:HumanBrainCount() == 0 then
		AudioMan:ClearMusicQueue();
		AudioMan:PlayMusic("Base.rte/Music/dBSoundworks/udiedfinal.ogg", 2, -1.0);
		AudioMan:QueueSilence(10);
		AudioMan:QueueMusicStream("Base.rte/Music/dBSoundworks/ccambient4.ogg");		
	else
		-- But if humans are left, then play happy music!
		AudioMan:ClearMusicQueue();
		AudioMan:PlayMusic("Base.rte/Music/dBSoundworks/uwinfinal.ogg", 2, -1.0);
		AudioMan:QueueSilence(10);
		AudioMan:QueueMusicStream("Base.rte/Music/dBSoundworks/ccambient4.ogg");
	end
end


function OneManArmy:UpdateActivity()
	if self.ActivityState ~= Activity.OVER then
		ActivityMan:GetActivity():SetTeamFunds(0,0)
		for player = Activity.PLAYER_1, Activity.MAXPLAYERCOUNT - 1 do
			if self:PlayerActive(player) and self:PlayerHuman(player) then
				--Display messages.

				local time = math.floor(self.SurvivalTimer:LeftTillSimMS(self.TimeLimit)/1000);
				local mins = 1+math.floor(time/60);
				local secs = time-(mins-1)*60;

				if self.StartTimer:IsPastSimMS(3000) then
					FrameMan:SetScreenText(mins-1 .. " minutes " .. secs .. " seconds | " .. self:GetTeamDeathCount(self.CPUTeam) .. " kills", player, 0, 1000, false)
					--FrameMan:SetScreenText(math.floor(self.SurvivalTimer:LeftTillSimMS(self.TimeLimit) / 1000) .. " seconds left", player, 0, 1000, false)
				else
					FrameMan:SetScreenText("Survive for " .. self.timeDisplay .. "!", player, 333, 5000, true)
				end
			
				-- The current player's team
				local team = self:GetTeamOfPlayer(player)
				-- Check if any player's brain is dead
				local brain = self:GetPlayerBrain(player)
				if not MovableMan:IsActor(brain) then
					self:SetPlayerBrain(nil, player)
					self:ResetMessageTimer(player)
					FrameMan:ClearScreenText(player)
					FrameMan:SetScreenText("Your brain has been destroyed!", player, 333, -1, false)
					-- Now see if all brains of self player's team are dead, and if so, end the game
					if not MovableMan:GetFirstBrainActor(team) then
						self.WinnerTeam = self:OtherTeam(team)
						ActivityMan:EndActivity()
					end
				else
					self.HuntPlayer = player
					local controller = brain:GetController();
					brain.Vel.X = controller:IsState(Controller.MOVE_RIGHT) and brain.Vel.X + 1 / (1 + math.abs(brain.Vel.Magnitude)) or brain.Vel.X;
					brain.Vel.X = controller:IsState(Controller.MOVE_LEFT) and brain.Vel.X - 1 / (1 + math.abs(brain.Vel.Magnitude)) or brain.Vel.X;
				end
				
				--Check if the player has won.
				if self.SurvivalTimer:IsPastSimMS(self.TimeLimit) then
					self:ResetMessageTimer(player)
					FrameMan:ClearScreenText(player)
					FrameMan:SetScreenText("You survived!", player, 333, -1, false)
					
					self.WinnerTeam = player
					
					--Kill all enemies.
					for actor in MovableMan.Actors do
						if actor.Team ~= self.WinnerTeam then
							actor.Health = 0
						end
					end

					ActivityMan:EndActivity()
				end
			end
		end

		--Spawn the AI.
		if self.CPUTeam ~= Activity.NOTEAM and self.ESpawnTimer:LeftTillSimMS(self.TimeLeft) <= 0 and MovableMan:GetTeamMOIDCount(self.CPUTeam) <= rte.AIMOIDMax * 3 / self:GetActiveCPUTeamCount() then
			local ship, actorsInCargo
		
			-- Set up the ship to deliver this stuff
			ship = CreateACRocket("Paradrop", "4zK.rte");
			actorsInCargo = 1;	--math.min(ship.MaxPassengers, 2)
			
			ship.Team = self.CPUTeam;
			
			-- Set the ship up with a cargo of a few armed and equipped actors
			for i = 1, actorsInCargo do
				-- Get any Actor from the CPU's native tech
				local passenger = RandomAHuman("Actors - Light", self.CPUTechName);

				for i = 1, 2 do
					local glove = CreateHDFirearm(self.enemyGlove);
					glove.HUDVisible = false;
					glove.GibImpulseLimit = math.random(100);
					passenger:AddInventoryItem(glove);
				end
				if math.random(100) < self.Difficulty then
					passenger:AddInventoryItem(CreateHDFirearm("Medikit"));
				end
				-- Set AI mode and team so it knows who and what to fight for!
				passenger.AIMode = Actor.AIMODE_BRAINHUNT;
				passenger.Team = self.CPUTeam;

				-- Check that we can afford to buy and to carry the weight of this passenger
				if (ship:GetTotalValue(0,3) + passenger:GetTotalValue(0,3)) <= self:GetTeamFunds(self.CPUTeam) and ship.InventoryMass <= ship.MaxInventoryMass then
					-- Yes we can; so add it to the cargo hold
					ship:AddInventoryItem(passenger);
					passenger = nil;
				else
					-- Nope; just delete the nixed passenger and stop adding new ones
					-- This doesn't need to be explicitly deleted here, teh garbage collection would do it eventually..
					-- but since we're so sure we don't need it, might as well go ahead and do it here right away
					DeleteEntity(passenger);
					passenger = nil;
					
					if i < 2 then	-- Don't deliver empty craft
						DeleteEntity(ship);
						ship = nil;
					end
					
					break;
				end
			end
			
			if ship then
				if SceneMan.SceneWrapsX then
					ship.Pos = Vector(math.random() * SceneMan.SceneWidth, -50);
				else
					ship.Pos = Vector(RangeRand(100, SceneMan.SceneWidth-100), -50);
				end
				-- Set the spawn point of the ship from orbit
				for player = Activity.PLAYER_1, Activity.MAXPLAYERCOUNT - 1 do
					if self:PlayerActive(player) and self:PlayerHuman(player) then
						local brain = self:GetPlayerBrain(player);
						if brain then
							ship.Pos = Vector(brain.Pos.X + math.random(-100, 100), -50);
							break;
						end
					end
				end

				-- Double-check if the computer can afford this ship and cargo, then subtract the total value from the team's funds
				local shipValue = ship:GetTotalValue(0,3)
				if shipValue <= self:GetTeamFunds(self.CPUTeam) then
					-- Subtract the total value of the ship+cargo from the CPU team's funds
					self:ChangeTeamFunds(-shipValue, self.CPUTeam);
					-- Spawn the ship onto the scene
					MovableMan:AddActor(ship);
				else
					-- The ship and its contents is deleted if it can't be afforded
					-- This doesn't need to be explicitly deleted here, teh garbage collection would do it eventually..
					-- but since we're so sure we don't need it, might as well go ahead and do it here right away
					DeleteEntity(ship);
					ship = nil;
				end
			end

			self.ESpawnTimer:Reset();
			self.TimeLeft = (self.BaseSpawnTime + MovableMan:GetTeamMOIDCount(self.CPUTeam)*10 + math.random(self.RandomSpawnTime) * rte.SpawnIntervalScale)
			self.BaseSpawnTime = self.BaseSpawnTime*0.96;
		end
	end
end
