function Create(self)
	self.updateTimer = Timer();

	self.lungeTapDelay = 200;
	self.lungePower = 6;
	self.tapTimer = Timer();
	
	self.prevHFlipped = self.HFlipped;
	self.speed = self:GetLimbPathSpeed(1);
	self.pushForce = self.LimbPathPushForce;
	self.stableVel = self:GetStableVelocityThreshold();
	
	function self.lunge(power)
		local flip = 0;
		if self.Status == Actor.STABLE then
			flip = self.FlipFactor;
			if self.controller:IsState(Controller.MOVE_RIGHT) then
				flip = 1;
			elseif self.controller:IsState(Controller.MOVE_LEFT) then
				flip = -1;
			end
			--Different factors that affect the lunge
			local angVel = math.abs(self.AngularVel * 0.1) + 1;
			local vel = (self.Vel.Magnitude + angVel)^2 * 0.0005 + 1;
			local mass = math.abs(self.Mass * 0.005) + 1;
			local aimAng = self:GetAimAngle(false);
			local vertical = math.abs(math.cos(aimAng))/vel;
			local strength = power * math.min(self.Health/self.MaxHealth, 1);
			
			local jumpVec =	Vector((power + strength/vel) * flip, -(power * 0.5 + (strength * 0.3)) * vertical):RadRotate(aimAng * self.FlipFactor);
			
			self.Vel = self.Vel + jumpVec/mass;
			self.AngularVel = self.AngularVel - (1/angVel * vertical) * flip * math.cos(self.RotAngle);
			self.Status = Actor.UNSTABLE;
			self.tapTimer:Reset();
		end
		return flip;
	end
end
function Update(self)
	self.controller = self:GetController();
	if self.updateTimer:IsPastSimMS(1000) then
		self.updateTimer:Reset();
		self.aggressive = self.Health < self.MaxHealth * 0.33;
	end
	
	local crouching = false;
	if self:IsPlayerControlled() and self.Status < Actor.INACTIVE and self.FGLeg and self.BGLeg then
		crouching = self.controller:IsState(Controller.BODY_CROUCH);
		if self.Status == Actor.UNSTABLE then
			if self.dir == self.FlipFactor then
				local motion = (self.Vel.Magnitude * 0.5 + math.abs(self.AngularVel));
				local stillness = 1/(1 + motion);
				if crouching then
					self.AngularVel = self.AngularVel * (1 - stillness) - (self.RotAngle - (self:GetAimAngle(false) - math.pi * 0.5) * self.FlipFactor) * 2 * stillness;
				end
			else
				self.dir = 0;
			end
			crouching = false;
		elseif crouching then
			if not self.crouchHeld then
				if not self.tapTimer:IsPastSimMS(self.lungeTapDelay) and SceneMan.Scene.GlobalAcc.Magnitude > 10 then
					self.dir = self.lunge(self.lungePower);
				end
				self.tapTimer:Reset();
				self.controller:SetState(Controller.BODY_CROUCH, false);
			end
		end
		self.crouchHeld = crouching;
	end
	if self.prevHFlipped ~= self.HFlipped and not self.flipTimer then
		--Turn
		self.flippedLimbs = {self.FGArm, self.BGArm, self.FGLeg, self.BGLeg};
		for _, limb in pairs(self.flippedLimbs) do
			if limb then
				limb.ParentOffset.X = limb.ParentOffset.X * 0.5;
			end
		end
		self.flippedParts = {self.Jetpack, self.Head};
		for _, part in pairs(self.flippedParts) do
			if part then
				part.ParentOffset.X = part.ParentOffset.X * 0.5;
				part.Frame = 1;	--Middle frame
			end
		end
		self.Frame = 1;
		self.flipTimer = Timer();
	elseif self.flipTimer and self.flipTimer:IsPastSimMS(40) then
		for _, limb in pairs(self.flippedLimbs) do
			if limb then
				limb.ParentOffset.X = limb.ParentOffset.X * 2;
			end
		end
		for _, part in pairs(self.flippedParts) do
			if part then
				part.ParentOffset.X = part.ParentOffset.X * 2;
				part.Frame = 0;
			end
		end
		self.Frame = 0;
		self.flippedLimbs = nil;
		self.flippedParts = nil;
		self.flipTimer = nil;
	end
	self:SetStableVelocityThreshold(crouching and self.stableVel * (1 + math.cos(self.RotAngle)) or self.stableVel)
	self.prevHFlipped = self.HFlipped;
end
function OnStride(self)--, layer)
	self:SetLimbPathSpeed(1, UInputMan.FlagShiftState and self.speed * 1.5 or self.speed);
	self.LimbPathPushForce = UInputMan.FlagShiftState and self.pushForce * 1.2 or self.pushForce;
	local angle = math.pi * 0.25 * self.FlipFactor;
	--self:SetWalkAngle(layer, angle);
--[[
	local hitPos = Vector();
	local radius = self.IndividualRadius * 2;
	local magnitude = radius;
	local checkPos = self.Pos + Vector(0, self.IndividualRadius * 0.5);
	if SceneMan:CastStrengthRay(checkPos, Vector(radius * self.FlipFactor, 0), 20, hitPos, 4, rte.grassID, SceneMan.SceneWrapsX) then
		local ratio = 0.5;
		self:SetLimbPathSpeed(1, self.origSpeed * ((1 - ratio) + ratio * math.max(SceneMan:ShortestDistance(checkPos, hitPos, SceneMan.SceneWrapsX).Magnitude/radius)));
	else
		self:SetLimbPathSpeed(1, self.origSpeed);
	end
]]--
end
function OnCollideWithMO(self, mo, rootMO)
	local forceVector = SceneMan:ShortestDistance(self.Pos, rootMO.Pos, SceneMan.SceneWrapsX):SetMagnitude(self.Mass * math.sqrt(1 + self.Vel.Magnitude));
	rootMO:AddForce(forceVector, Vector());
	if self.Mass > rootMO.Mass and IsActor(rootMO) then ToActor(rootMO).Status = Actor.UNSTABLE; end
end