function Create(self)
	self.lungeTapDelay = 200;
	self.frontFlip = {input = {Controller.BODY_JUMP, Controller.BODY_CROUCH}, inputCounter = 0};
	self.lungePower = 7;
	self.tapTimer = Timer();
	
	self.jumpSound = CreateSoundContainer("Ninja Jump", "4zK.rte");
	function self.lunge(power)
		local flip = 0;
		if self.Status == Actor.STABLE then
			flip = self.FlipFactor;
			if self.controller:IsState(Controller.MOVE_RIGHT) then
				flip = 1;
			elseif self.controller:IsState(Controller.MOVE_LEFT) then
				flip = -1;
			end
			--Different factors that affect the lunge
			local angVel = math.abs(self.AngularVel * 0.1) + 1;
			local vel = (self.Vel.Magnitude + angVel)^2 * 0.0005 + 1;
			local mass = math.abs(self.Mass * 0.005) + 1;
			local aimAng = self:GetAimAngle(false);
			local vertical = math.abs(math.cos(aimAng))/vel;
			local strength = power * math.min(self.Health/self.MaxHealth, 1);
			
			local jumpVec =	Vector((power + strength/vel) * flip, -(power * 0.5 + (strength * 0.3)) * vertical):RadRotate(aimAng * self.FlipFactor);
			
			self.Vel = self.Vel + jumpVec/mass;
			self.AngularVel = self.AngularVel - (1/angVel * vertical) * flip * math.cos(self.RotAngle);
			self.Status = Actor.UNSTABLE;
			self.tapTimer:Reset();
		end
		return flip;
	end
end
function Update(self)
	self.controller = self:GetController();
	local crouching = self.controller:IsState(Controller.BODY_CROUCH);
	if self.jumpSound then
		self.jumpSound.Pos = self.Pos;
	end
	if self:IsPlayerControlled() and self.Status < Actor.DYING and self.FGLeg and self.BGLeg then
		if self.Status == Actor.UNSTABLE then
			if not crouching then
				local motion = self.Vel.Magnitude * 0.5 + math.abs(self.AngularVel);
				if motion > 1 then
					self.AngularVel = self.AngularVel * (1 - 0.1/motion);
				end
				if self.tapTimer:IsPastSimMS(500) and math.cos(self.RotAngle) > 0.9 and motion < 20  then
					self.tapTimer:Reset();
					self.Status = Actor.STABLE;
					self.AngularVel = self.AngularVel * 0.5;
				end
			end
		else
			--Track the key combo input (jump -> crouch) to trigger frontflip
			if self.controller:IsState(self.frontFlip.input[self.frontFlip.inputCounter + 1]) then
				if (self.frontFlip.inputCounter == 0 or not self.tapTimer:IsPastSimMS(self.lungeTapDelay)) then
					self.frontFlip.inputCounter = self.frontFlip.inputCounter + 1;
					if self.frontFlip.inputCounter == #self.frontFlip.input then
						self.AngularVel = self.AngularVel - (self.lunge(self.lungePower) * 8);
						self.frontFlip.inputCounter = 0;
						self.jumpSound:Play(self.Pos);
					end
					self.tapTimer:Reset();
				else
					self.frontFlip.inputCounter = 0;
				end
			elseif crouching then
				if not self.keyHeld then
					if not self.tapTimer:IsPastSimMS(self.lungeTapDelay) and SceneMan.Scene.GlobalAcc.Magnitude > 10 then
						self.lunge(self.lungePower);
					end
					self.tapTimer:Reset();
				end
				self.keyHeld = true;
			else
				self.keyHeld = false;
			end
		end
	end
end