function Create(self)
	self.mapWrapsX = SceneMan.SceneWrapsX;
	self.climbTimer = Timer();
	self.mouseClimbTimer = Timer();
	self.actionMode = 0;	-- 0 = start, 1 = flying, 2 = grab terrain, 3 = grab MO
	self.climb = 0;
	self.canRelease = false;

	self.tapTimer = Timer();
	self.tapCounter = 0;
	self.didTap = false;
	self.canTap = false;

	self.maxLineLength = 600;
	self.setLineLength = 0;
	self.lineStrength = 40;	-- How much "force" the rope can take before breaking
	
	self.limitReached = false;
	self.stretchPullRatio = 0.1;

	self.tapTime = 150;	-- Maximum amount of time between tapping for claw to return
	self.tapAmount = 2;	-- How many times to tap to bring back rope
	
	for i = 1, MovableMan:GetMOIDCount() - 1 do
		local webslinger = MovableMan:GetMOFromID(i);
		if webslinger and webslinger.ClassName == "AHuman" and webslinger.PresetName == "Spidey" and SceneMan:ShortestDistance(self.Pos, webslinger.Pos, self.mapWrapsX).Magnitude < 50 then
			self.parent = ToAHuman(webslinger);
			self.parentRadius = self.parent.Radius;
			if self.parent.Sharpness == 1 then
				self.webArm = self.parent.EquippedBGItem == nil and self.parent.BGArm or (self.parent.EquippedItem == nil and self.parent.FGArm or nil);
			else
				self.webArm = self.parent.EquippedItem == nil and self.parent.FGArm or (self.parent.EquippedBGItem == nil and self.parent.BGArm or nil);
			end
			self.Pos = self.webArm.Pos;
			self.Vel = Vector(60, 0):RadRotate(self.parent:GetAimAngle(true));
			self.parent.Sharpness = math.abs(self.parent.Sharpness - 1);
			self.actionMode = 1;
			break;
		end
	end
	if self.parent == nil then	-- Failed to find our gun, abort
		self.ToDelete = true;
	end
	self.holdTimer = Timer();
end
function Update(self)
	if self.parent and IsAHuman(self.parent) and IsArm(self.webArm) then
		local controller = self.parent:GetController();
		local startPos = ToArm(self.webArm).HandPos;
		
		--self.ToDelete = false;
		self.ToSettle = false;

		self.lineVec = SceneMan:ShortestDistance(self.parent.Pos, self.Pos, self.mapWrapsX);
		self.lineLength = self.lineVec.Magnitude;

		if IsAHuman(self.parent) then
			self.parent = ToAHuman(self.parent);
			-- We now have a user that controls this grapple
			controller = self.parent:GetController();
			-- Point the gun towards the hook if our user is holding it
			local offset = Vector(self.lineLength, 0):RadRotate(self.parent.FlipFactor * (self.lineVec.AbsRadAngle) + (self.parent.HFlipped and math.pi or 0));
			self.webArm.IdleOffset = offset;
			-- Prevent the user from spinning like crazy
			if self.parent.Status > Actor.STABLE then
				self.parent.AngularVel = self.parent.AngularVel/(1 + math.abs(self.parent.AngularVel) * 0.01);
			end
		end

		self.lastSetLineLength = self.setLineLength;

		if self.actionMode == 1 then	-- Hook is in flight
			self.rayVec = Vector();
			-- Stretch mode: gradually retract the hook for a return hit
			if self.limitReached then
				self.Vel = self.Vel - Vector(self.lineVec.X, self.lineVec.Y):SetMagnitude(math.sqrt(self.lineLength) * self.stretchPullRatio/2);
			end
			-- Detect terrain and stick if found
			local ray = Vector(self.Radius + self.Vel.Magnitude * rte.PxTravelledPerFrame, 0):RadRotate(self.Vel.AbsRadAngle);
			
			local hitPos = Vector();
			
			local obstRay = SceneMan:CastObstacleRay(self.Pos, ray, hitPos, Vector(), self.parent.ID, -2, rte.airID, 3);
			if obstRay >= 0 then
				self.Pos = hitPos;
				local id = SceneMan:GetMOIDPixel(hitPos.X, hitPos.Y);
				if id ~= rte.NoMOID and id ~= self.parent.ID then
					self.target = MovableMan:GetMOFromID(id);
					-- Treat pinned MOs as terrain
					if self.target.PinStrength > 0 then
						self.actionMode = 2;
					else
						self.stickPosition = SceneMan:ShortestDistance(self.target.Pos, self.Pos, self.mapWrapsX);
						self.stickRotation = self.target.RotAngle;
						self.stickDirection = self.RotAngle;
						self.actionMode = 3;
					end
					local actor = ToMOSRotating(self.target):GetRootParent();
					if actor.Team ~= self.Team and actor.Mass < self.parent.Mass * 2 and IsActor(actor) and ToActor(actor).Status == Actor.STABLE then
						ToActor(actor).Status = Actor.UNSTABLE;
					end
				else	-- Hit terrain
					self.actionMode = 2;
				end
			elseif self.Pos.Y < 0 then
				self.actionMode = 2;
				self.Lifetime = self.Age + 1000/(1 + math.sqrt(self.lineLength) * 0.1);
			end
			if self.actionMode > 1 then
				self.setLineLength = math.floor(self.lineLength);
				self.Vel = Vector();
				self.PinStrength = 1000;
				self.Frame = 1;
				self.lastVel = Vector(self.Pos.X, self.Pos.Y);
			end
			if self.lineLength > self.maxLineLength then
				if self.limitReached == false then
					self.limitReached = true;
				end
				local movetopos = self.parent.Pos + (self.lineVec):SetMagnitude(self.maxLineLength);
				if self.mapWrapsX == true then
					if movetopos.X > SceneMan.SceneWidth then
						movetopos = Vector(movetopos.X - SceneMan.SceneWidth, movetopos.Y);
					elseif movetopos.X < 0 then
						movetopos = Vector(SceneMan.SceneWidth + movetopos.X, movetopos.Y);
					end
				end
				self.Pos = movetopos;

				local pullamountnumber = math.abs(-self.lineVec.AbsRadAngle + self.Vel.AbsRadAngle)/6.28;
				self.Vel = self.Vel - self.lineVec:SetMagnitude(self.Vel.Magnitude * pullamountnumber);
			end
		elseif self.actionMode > 1 then	-- Hook has stuck
			-- Actor mass and velocity affect pull strength negatively, rope length affects positively (diminishes the former)
			local parentForces = 1 + (self.parent.Vel.Magnitude * 5 + self.parent.Mass)/(1 + self.lineLength);
			if self.actionMode == 2 then	-- Stuck terrain
				if controller:IsState(Controller.BODY_CROUCH) and self.parent.Status < Actor.INACTIVE then
					self.parent.Status = Actor.UNSTABLE;
				end
				local pullVec = self.lineVec:SetMagnitude(self.stretchPullRatio * math.sqrt(self.lineLength)/parentForces);
				self.parent.Vel = self.parent.Vel + pullVec;
				
			elseif self.actionMode == 3 then	-- Stuck MO
				if self.target.ID ~= rte.NoMOID then

					self.Pos = self.target.Pos + Vector(self.stickPosition.X, self.stickPosition.Y):RadRotate(self.target.RotAngle - self.stickRotation);
					self.RotAngle = self.stickDirection + (self.target.RotAngle - self.stickRotation);

					local jointStiffness;
					local target = self.target;
					if target.ID ~= target.RootID then
						local mo = target:GetRootParent();
						if mo.ID ~= rte.NoMOID and IsAttachable(target) then
							-- It's best to apply all the forces to the parent instead of utilizing JointStiffness
							target = mo;
						end
					end
					local factor = 1.5;

					local pullVec = self.lineVec:SetMagnitude(self.stretchPullRatio * math.sqrt(self.lineLength)/parentForces);
					self.parent.Vel = self.parent.Vel + pullVec/factor;

					local targetForces = 1 + (target.Vel.Magnitude * 5 + target.Mass)/(1 + self.lineLength);
					target.Vel = target.Vel - (pullVec * factor) * parentForces/targetForces;
				else	-- Our MO has been destroyed, return hook
					self.ToDelete = true;
				end
			end
		end
		local fire = controller:IsState(Controller.WEAPON_FIRE);
		if not fire then
			self.holdTimer = nil;
		elseif self.holdTimer and self.holdTimer:IsPastSimMS(250) then
			self.autoRelease = true;
		end
		if (controller:IsState(Controller.BODY_JUMP) and self.actionMode == 2) or (not fire and self.autoRelease) then
			self.ToDelete = true;
		end
		-- Fine tuning: take the seam into account when drawing the rope
		local drawPos = self.parent.Pos + self.lineVec:SetMagnitude(self.lineLength);
		if self.ToDelete == true then
			drawPos = self.parent.Pos + (self.lineVec * 0.5);
		end
		PrimitiveMan:DrawLinePrimitive(startPos, drawPos, 252);
	else
		self.ToDelete = true;
	end
end