function Create(self)
	self.damageMultiplier = 2;
	self.width = self.Radius * 0.5;
	
	self.hitSound = CreateSoundContainer("Arrow Hit", "4zK.rte");
end
function Update(self)
	if self.hitThisFrame then
		self.hitThisFrame = false;
	end
end
function OnCollideWithMO(self, mo, rootMO)
	if not (self.target or self.hitThisFrame) then
		local trajectoryScalar = math.abs(math.cos(self.RotAngle - self.Vel.AbsRadAngle));
		
		local hitPos = Vector();
		local dots = math.sqrt(self.PrevVel.Magnitude);
		local trace = Vector((self.PrevVel.Magnitude * rte.PxTravelledPerFrame + self.width) * trajectoryScalar * self.FlipFactor, 0):RadRotate(self.PrevRotAngle);
		for i = 1, dots do
			local checkPos = self.PrevPos + trace * (i/dots);
			local checkPix = SceneMan:GetMOIDPixel(checkPos.X, checkPos.Y);
			if checkPix ~= rte.NoMOID then
				hitPos = Vector(checkPos.X, checkPos.Y);
				local foundMO = ToMOSRotating(MovableMan:GetMOFromID(checkPix));
				local pen = (self.Mass * self.PrevVel.Magnitude * self.Sharpness)/math.max(foundMO.Material.StructuralIntegrity, 1);
				if pen > 1 then
					checkPos = checkPos + SceneMan:ShortestDistance(checkPos, self.PrevPos, SceneMan.SceneWrapsX):SetMagnitude(self.width);

					local moAngle = -foundMO.RotAngle * mo.FlipFactor;
					local dist = SceneMan:ShortestDistance(foundMO.Pos, checkPos, SceneMan.SceneWrapsX);
					local stickOffset = Vector(dist.X * foundMO.FlipFactor, dist.Y):RadRotate(-foundMO.RotAngle * foundMO.FlipFactor);
					
					local woundName = foundMO:GetEntryWoundPresetName();
					if woundName ~= "" then
						local wound = CreateAEmitter(woundName);
						wound.DamageMultiplier = self.damageMultiplier;
						wound.EmitCountLimit = math.ceil(wound.EmitCountLimit * 0.75);
						wound.Scale = wound.Scale * 0.75;
						for em in wound.Emissions do
							em.ParticlesPerMinute = em.ParticlesPerMinute * 0.75;
							em.BurstSize = em.BurstSize * 0.75;
							em.MaxVelocity = em.MaxVelocity * 0.75;
							em.MinVelocity = em.MinVelocity * 0.75;
						end
						wound.InheritedRotAngleOffset = stickOffset.AbsRadAngle - (foundMO.HFlipped and math.pi or 0);
						foundMO:AddWound(wound, Vector(stickOffset.X, stickOffset.Y):SetMagnitude(stickOffset.Magnitude - self.width), math.random() < 0.5);
					end
					if pen > 2 and self.PrevVel.Magnitude * rte.PxTravelledPerFrame > foundMO.Radius * 1.5 then
						local parent = nil;
						local skipPx = 2;
						local traceIncrement = Vector(trace.X, trace.Y):SetMagnitude(skipPx);
						for j = 0, foundMO.Diameter/(1 + skipPx) do
							hitPos = hitPos + traceIncrement;
							local noID = SceneMan:GetMOIDPixel(hitPos.X, hitPos.Y);
							if noID == rte.NoMOID or noID ~= checkPix then
								--Move one step down in attachable hierarchy
								if not parent and foundMO:GetParent() and noID == foundMO:GetParent().ID then
									parent = foundMO:GetParent();
									foundMO = parent;
									checkPix = parent.ID;
								else
									local woundName = foundMO:GetExitWoundPresetName();
									if woundName ~= "" then
										local wound = CreateAEmitter(woundName);
										wound.DamageMultiplier = self.damageMultiplier;
										wound.EmitCountLimit = math.ceil(wound.EmitCountLimit * 1.25);
										wound.Scale = wound.Scale * 1.25;
										for em in wound.Emissions do
											em.ParticlesPerMinute = em.ParticlesPerMinute * 1.25;
											em.BurstSize = em.BurstSize * 1.25;
											em.MaxVelocity = em.MaxVelocity * 1.25;
											em.MinVelocity = em.MinVelocity * 1.25;
										end
										local dist = SceneMan:ShortestDistance(foundMO.Pos, hitPos - traceIncrement, SceneMan.SceneWrapsX);
										local offset = Vector(dist.X * foundMO.FlipFactor, dist.Y):RadRotate(moAngle):SetMagnitude(dist.Magnitude - (wound.Radius - 1) * wound.Scale);
										local traceOffset = Vector(trace.X * foundMO.FlipFactor, trace.Y):RadRotate(moAngle);
										
										wound.InheritedRotAngleOffset = traceOffset.AbsRadAngle - (foundMO.HFlipped and math.pi or 0);
										foundMO:AddWound(wound, Vector(offset.X, offset.Y), math.random() < 0.5);
										
										--This doesn't work but it's kind of funny
										self:SetWhichMOToNotHit(foundMO, 1000);
										foundMO:SetWhichMOToNotHit(self, 1000);
										
										self.hitThisFrame = true;
										self.Pos = hitPos;
										self.Vel = self.PrevVel;
										self:ClearImpulseForces();
										self.RotAngle = self.PrevRotAngle;
									end
									break;
								end
							end
						end
					else
						self.target = foundMO;
						self.Team = self.target.Team;
						self.InheritsHFlipped = (self.HFlipped == self.target.HFlipped) and 1 or -1;
						self.DrawAfterParent = math.random() < 0.5;
						
						self.InheritedRotAngleOffset = (self.PrevRotAngle - self.target.RotAngle) * self.target.FlipFactor;
						self.target:AddAttachable(self:Clone(), stickOffset);
						
						self.hitSound:Play(self.Pos);
						
						self.ToDelete = true;
					end
				end
				break;
			end
		end
	end
end
function OnAttach(self, parent)
	self.target = parent;
end
function OnDetach(self, parent)
	self.target = nil;
end