function Create(self)
	self.lastVel = Vector(self.Vel.X, self.Vel.Y);
	
	self.lifeTimer = Timer();
	self.gibTime = 5000;
end
function Update(self)
	if self.PinStrength ~= 0 then
		if self.target and self.target.ID ~= rte.NoMOID and not self.target.ToDelete then
			self.Vel = Vector();
			self.AngularVel = 0;

			self.Pos = self.target.Pos - Vector(self.stickPos.X, self.stickPos.Y):RadRotate(self.target.RotAngle - self.targetStickAngle);
			self.RotAngle = self.stickAngle + (self.target.RotAngle - self.targetStickAngle);
		else
			self.Vel = self.Vel + Vector(math.random(), 0):RadRotate(6.28 * math.random());
			self.AngularVel = self.AngularVel + math.random(-5, 5);
			self.PinStrength = 0;
		end
	else
		self.target = nil;

		if self.lastVel.Magnitude > 10 then
			local riccochet = true;

			local dots = self.Vel.Magnitude * rte.PxTravelledPerFrame + 1;
			local trace = Vector(dots * self.FlipFactor, 0):RadRotate(self.RotAngle);
			local hitPos = Vector(self.Pos.X, self.Pos.Y);
			local obstRay = SceneMan:CastObstacleRay(self.Pos, trace, hitPos, Vector(), self.ID, self.Team, rte.airID, dots * 0.2);
			if obstRay >= 0 then
				local penetration = self.Vel.Magnitude * self.Mass * self.Sharpness;
				local moCheck = SceneMan:GetMOIDPixel(hitPos.X, hitPos.Y);
				if moCheck == rte.NoMOID then	-- hit terrain
					local matStrength = SceneMan:GetMaterialFromID(SceneMan:GetTerrMatter(hitPos.X, hitPos.Y)).StructuralIntegrity;
					local depth = (penetration * 0.5) - matStrength;
					if depth > 1 then
						if matStrength < penetration then	-- stick to terrain
							self.Vel = Vector();
							self.Pos = self.Pos + Vector(trace.X, trace.Y):SetMagnitude(obstRay - self.Radius/depth);
							self.PinStrength = matStrength;
							self.ToSettle = true;
							riccochet = false;	-- don't play riccochet sound when sticking
						end
					else
						riccochet = false;	-- don't play riccochet sound when sinking into weak terrain
					end
				else
					local mo = MovableMan:GetMOFromID(moCheck);
					local depth = penetration - mo.Material.StructuralIntegrity;
					if mo and depth > 1 then

						self.target = ToMOSRotating(mo);
						self.PinStrength = self.target.Material.StructuralIntegrity;

						self.stickAngle = self.RotAngle;	-- own angle when sticking
						self.targetStickAngle = self.target.RotAngle;		-- target angle when sticking

						self.Pos = self.Pos + Vector(trace.X, trace.Y):SetMagnitude(obstRay - self.Radius/depth);	-- close the distance
						self.stickPos = SceneMan:ShortestDistance(self.Pos, self.target.Pos, SceneMan.SceneWrapsX);
						-- add damage particle
						for i = 1, 2 do
							local part = CreateMOPixel("4zK.rte/Arrow Damage Particle");
							part.Pos = self.Pos;
							part.Vel = self.Vel;
							part.Mass = self.Mass/i;
							part.Sharpness = self.Sharpness;
							part.WoundDamageMultiplier = math.sqrt(1 + depth) * 0.3;
							MovableMan:AddParticle(part);
						end
						riccochet = false;	-- don't play riccochet sound when sticking
					end
				end
			end
			if riccochet then
				local ang = 0.1;	-- trajectory angle change to trigger riccochet sound, in rad
				if math.abs(self.lastVel.AbsRadAngle - self.Vel.AbsRadAngle) > ang and math.random() < (self.lastVel.Magnitude/10) then
					--AudioMan:PlaySound("Base.rte/Sounds/Penetration/Riccochet".. math.random(5) ..".wav", self.Pos);
				end
			end
			self.lifeTimer:Reset();
		elseif self.lifeTimer:IsPastSimMS(self.gibTime) then
			self:GibThis();
		end
	end
	self.lastVel = Vector(self.Vel.X, self.Vel.Y);
	if self.Age > (self.gibTime * 5) then
		self:GibThis();
	end
end