
function Create(self)
	self.cursorSpeed = 2;
	self.screen = CreateMOSRotating("Hitman Screen");
	self.activity = ActivityMan:GetActivity();
	self.stance = Vector(7, 5);
	self.sharpStance = Vector(8, -1);
end

function Update(self)

	local dist = Vector(self.FlipFactor, 0):RadRotate(self.RotAngle);
	local startPos = self.MuzzlePos;
	local parent = self:GetRootParent();
	if IsActor(parent) then
		parent = ToActor(parent);
		local ctrl = parent:GetController();

		local playerControlled = parent:IsPlayerControlled();
		local mouseControlled = ctrl:IsMouseControlled();
		local aiming = ctrl:IsState(Controller.AIM_SHARP);
		local jumping = ctrl:IsState(Controller.BODY_JUMP);
		local crouching = ctrl:IsState(Controller.BODY_CROUCH);
	
		if playerControlled and aiming and not self:IsReloading() then
			startPos = parent.EyePos;
			self.StanceOffset = self.sharpStance;
			self.HUDVisible = false;
			if not self.cursor then
				self.cursor = Vector(self.MuzzlePos.X, self.MuzzlePos.Y);
				local length = SceneMan:CastObstacleRay(self.cursor, Vector(self.SharpLength, 0):RadRotate(parent:GetAimAngle(true)), Vector(), self.cursor, self.ID, self.Team, rte.airID, 5);
			end
			dist = SceneMan:ShortestDistance(startPos, self.cursor, SceneMan.SceneWrapsX);
			self.cursor = startPos + Vector(dist.X, dist.Y):SetMagnitude(math.min(dist.Magnitude, self.SharpLength));
			parent:SetAimAngle((dist.AbsRadAngle - (parent.HFlipped and math.pi or 0)) * self.FlipFactor);
			parent.HFlipped = dist.X < 0;
			parent.ViewPoint = self.cursor;

			local cursorMovement = Vector();
			
			if mouseControlled then
				cursorMovement = cursorMovement + ctrl.MouseMovement;
			else
				if ctrl:IsState(Controller.HOLD_UP) or jumping then
					cursorMovement = cursorMovement + Vector(0, -self.cursorSpeed);
				end
				if ctrl:IsState(Controller.HOLD_DOWN) or crouching then
					cursorMovement = cursorMovement + Vector(0, self.cursorSpeed);
				end
				if ctrl:IsState(Controller.HOLD_LEFT) then
					cursorMovement = cursorMovement + Vector(-self.cursorSpeed, 0);
				end
				if ctrl:IsState(Controller.HOLD_RIGHT) then
					cursorMovement = cursorMovement + Vector(self.cursorSpeed, 0);
				end
			end
			self.cursor = self.cursor + cursorMovement;
			
			if not SceneMan.SceneWrapsX then
				if self.cursor.X > SceneMan.SceneWidth then
					self.cursor.X = math.min(self.cursor.X, SceneMan.SceneWidth);
				end
				if self.cursor.X < 0 then
					self.cursor.X = math.max(self.cursor.X, 0);
				end
			end
			if not SceneMan.SceneWrapsY then
				if self.cursor.Y > SceneMan.SceneHeight then
					self.cursor.Y = math.min(self.cursor.Y, SceneMan.SceneHeight);
				end
				if self.cursor.Y < 0 then
					self.cursor.Y = math.max(self.cursor.Y, 0);
				end
			end
			
			ctrl:SetState(Controller.MOVE_UP, false);
			ctrl:SetState(Controller.MOVE_DOWN, false);
			ctrl:SetState(Controller.MOVE_LEFT, false);
			ctrl:SetState(Controller.MOVE_RIGHT, false);
			ctrl:SetState(Controller.AIM_UP, false);
			ctrl:SetState(Controller.AIM_DOWN, false);
			ctrl:SetState(Controller.AIM_SHARP, false);
		
			local screen = self.activity:ScreenOfPlayer(ctrl.Player);
			local scrollTarget = SceneMan:GetScrollTarget(screen);
			--SceneMan:SetScrollTarget(self.cursor, 1, SceneMan:ShortestDistance(startPos, self.cursor, false).Magnitude > dist.Magnitude, screen);
			SceneMan:SetScrollTarget(self.cursor, 1, false, screen);

			local width = FrameMan.PlayerScreenWidth * 0.5;
			local height = FrameMan.PlayerScreenHeight * 0.5;
			
			PrimitiveMan:DrawLinePrimitive(screen, self.cursor - Vector(width, 0), self.cursor - Vector(2, 0), 245);
			PrimitiveMan:DrawLinePrimitive(screen, self.cursor - Vector(0, height), self.cursor - Vector(0, 2), 245);
			
			PrimitiveMan:DrawLinePrimitive(screen, self.cursor + Vector(width, 0), self.cursor + Vector(2, 0), 245);
			PrimitiveMan:DrawLinePrimitive(screen, self.cursor + Vector(0, height), self.cursor + Vector(0, 2), 245);
			
			PrimitiveMan:DrawBitmapPrimitive(screen, self.cursor, self.screen, 0, 0);
			--PrimitiveMan:DrawBoxFillPrimitive(screen, self.cursor, self.cursor, 245);
			--PrimitiveMan:DrawBoxFillPrimitive(screen, self.cursor, self.cursor, 245);
		else
			self.cursor = nil;
			self.StanceOffset = self.stance;
		end
	end
	if self.FiredFrame then
		for i = 0, self.Sharpness do
			local vel = 450;
			local bullet = CreateMOPixel("Hitman Rifle Bullet");
			bullet.Pos = startPos + Vector(dist.X, dist.Y):SetMagnitude(math.max(dist.Magnitude - (vel * rte.PxTravelledPerFrame * 0.5) + i, 0));
			bullet.Vel = Vector(dist.X, dist.Y):SetMagnitude(vel);
			bullet:SetWhichMOToNotHit(parent, 100);
			MovableMan:AddParticle(bullet);
		end
	end
end