function Create(self)
	self.radius = self.PinStrength;
	self.origRadius = self.radius;
	self.PinStrength = 0;
	self.strength = 1;
	self.degradeIncrement = 0.02;

	if RiftReaperTable then
		self.tableNum = #RiftReaperTable + 1;
		RiftReaperTable[self.tableNum] = {id = self.UniqueID, pointer = self};
	else
		self.tableNum = 1;
		RiftReaperTable = {{id = self.UniqueID, pointer = self, radius = self.radius}};
	end
	self.spinDirection = math.random() < 0.5 and -1 or 1;
	
	self.checkTimer = Timer();
	self.checkTime = TimerMan.DeltaTimeMS;
	self.suckSound = CreateSoundContainer("Vortex Suck");
	self.suckSound:Play(self.Pos);
end
function Update(self)
	if self.PinStrength == 0 and self.Vel.Magnitude < 0.01 then
		self.PinStrength = self.origRadius;
	end
	local ratio = math.min(self.Age/1000, 1);
	self.suckSound.AttenuationStartDistance = 250 * ratio * (self.radius/self.origRadius);
	self.suckSound.Pitch = math.max(ratio, 0.1);
	local adjustedStrength = self.strength * ratio;
	local adjustedRadius = self.radius * ratio;

	RiftReaperTable[self.tableNum].radius = adjustedRadius;

	for _, neighbor in pairs(RiftReaperTable) do
		if neighbor and neighbor.id ~= self.UniqueID then
			local dist = SceneMan:ShortestDistance(self.Pos, neighbor.pointer.Pos, SceneMan.SceneWrapsX);
			local prox = dist.Magnitude - neighbor.radius;
			if prox < (adjustedRadius * 0.5) and neighbor.radius < self.radius then
				neighbor.pointer.ToDelete = true;
				self.radius = self.radius + (neighbor.radius * 0.5);

			elseif prox < (self.radius * adjustedStrength) then
				local move = dist:SetMagnitude((adjustedStrength * (neighbor.radius/self.radius))/math.sqrt(1 + dist.Magnitude));
				self.Pos = self.Pos + move;
			end
		end
	end

	if self.checkTimer:IsPastSimMS(self.checkTime) then
		self.checkTimer:Reset();
		local list = {MovableMan.Actors, MovableMan.Items, MovableMan.Particles};
		local dots = math.sqrt(adjustedRadius);
		
		local strength = (TimerMan.DeltaTimeMS + self.checkTime)/TimerMan.DeltaTimeMS * (self.radius * adjustedStrength);

		for _, coll in pairs(list) do
			for mo in coll do
				if mo and mo.PresetName ~= "RIFTREAPER" then
					local dist = SceneMan:ShortestDistance(self.Pos, mo.Pos, SceneMan.SceneWrapsX);
					if dist.Magnitude < (self.radius * adjustedStrength) and mo.PinStrength == 0 then
						local distFactor = 1 + math.sqrt(dist.Magnitude);
						local sizeFactor =  math.sqrt(mo.Mass + mo.Radius);
						local velFactor = math.sqrt(mo.Vel.Magnitude);
						local resistance = distFactor + sizeFactor + velFactor;
						local forceVector = Vector(-dist.X, -dist.Y):SetMagnitude(strength):RadRotate(1/distFactor * RangeRand(0, self.spinDirection));
						if IsMOSRotating(mo) then
							mo = ToMOSRotating(mo);
							for att in mo.Attachables do
								att:AddAbsForce(forceVector * 5 * math.sqrt(1 + math.abs(mo.Mass))/distFactor, self.Pos);
							end
							mo.AngularVel = (mo.AngularVel * 0.99) + (self.spinDirection * strength)/(math.abs(mo.AngularVel) + resistance);
						end
						mo.Vel = mo.Vel + (forceVector/resistance);
						dots = dots - (mo.Radius * 0.01);
					end
				end
			end
		end
		for i = 1, dots do
			local size = {"Tiny ", "Small ", ""};
			local rand = math.random(adjustedRadius);
			local smoke = CreateMOSParticle(size[math.random(rand > 100 and #size or (rand > 50 and 2 or 1))] .."Smoke Ball 1");
			local vector = Vector(adjustedRadius * RangeRand(0.3, 0.9), 0):RadRotate(6.28 * math.random());
			smoke.Pos = self.Pos + vector;
			smoke.Vel = vector * (-0.5);
			smoke.Lifetime = vector.Magnitude * TimerMan.DeltaTimeMS;
			MovableMan:AddParticle(smoke);
		end
	end
	self.radius = (self.radius * (1 - self.degradeIncrement/3)) - self.degradeIncrement;
	if self.radius < self.origRadius * (self.Age * 0.00001) then
		self.ToDelete = true;
	end
end
function Destroy(self)
	RiftReaperTable[self.tableNum] = nil;
	self.suckSound:Stop();
end