function Create(self)

	self.range = math.sqrt(FrameMan.PlayerScreenWidth^2 + FrameMan.PlayerScreenHeight^2) * 0.5;
	self.penet = 4;
	self.damageMultiplier = 4;

	self.shake = 0;
	self.origFireRate = self.RateOfFire;
	self.ricochetSound = CreateSoundContainer("Ricochet", "Base.rte");
end

function Update(self)

	if self.FiredFrame then
	
		local actor = self:GetRootParent();
		local aiming = false;
		if IsActor(actor) then
			actor = ToActor(actor);
			self.shake = self.shake + math.sqrt(actor.Vel.Magnitude) * 0.1;
			aiming = actor:GetController():IsState(Controller.AIM_SHARP);
			if aiming then
				self.shake = self.shake * 0.5;
			end
		end
		
		local range = self.range + (aiming and self.SharpLength or 0);
		
		local startPos = self.MuzzlePos;
		local hitPos = Vector(startPos.X, startPos.Y);
		local gapPos = Vector(startPos.X, startPos.Y);
		local skipPx = 2;
		local trace = Vector(range * self.FlipFactor, 0):RadRotate(self.RotAngle);
		--Use higher pixel skip to find a rough estimate
		local rayLength = SceneMan:CastObstacleRay(startPos, trace, hitPos, gapPos, actor.ID, self.Team, rte.airID, skipPx * 2);

		local traceDist = SceneMan:ShortestDistance(startPos, hitPos, SceneMan.SceneWrapsX);
		PrimitiveMan:DrawLinePrimitive(startPos + traceDist * math.random(), startPos + traceDist * math.random(), 118);
		
		if rayLength >= 0 then
		
			rayLength = SceneMan:CastObstacleRay(gapPos, Vector(trace.X, trace.Y):SetMagnitude(range - rayLength + 1), hitPos, gapPos, actor.ID, self.Team, rte.airID, skipPx);

			local moID = SceneMan:GetMOIDPixel(hitPos.X, hitPos.Y);
			if moID ~= rte.NoMOID and moID ~= self.ID then
				local mo = ToMOSRotating(MovableMan:GetMOFromID(moID));
				mo:AddImpulseForce(Vector(trace.X, trace.Y):SetMagnitude(100 * self.damageMultiplier), Vector());
				local moAngle = -mo.RotAngle * mo.FlipFactor;
				local diminishedMultiplier = math.sqrt(self.damageMultiplier);
				local woundName = mo:GetEntryWoundPresetName();
				if woundName ~= "" then
					local wound = CreateAEmitter(woundName);
					wound.EmitCountLimit = wound.EmitCountLimit * diminishedMultiplier;
					wound.BurstDamage = wound.BurstDamage * self.damageMultiplier;
					wound.Scale = wound.Scale * math.sqrt(diminishedMultiplier);
					for em in wound.Emissions do
						em.ParticlesPerMinute = em.ParticlesPerMinute * self.damageMultiplier;
						em.BurstSize = em.BurstSize * self.damageMultiplier;
						em.MaxVelocity = em.MaxVelocity * diminishedMultiplier;
						em.MinVelocity = em.MinVelocity * diminishedMultiplier;
					end
					local dist = SceneMan:ShortestDistance(mo.Pos, hitPos, SceneMan.SceneWrapsX);
					local offset = Vector(dist.X * mo.FlipFactor, dist.Y):RadRotate(moAngle):SetMagnitude(dist.Magnitude - (wound.Radius - 1) * wound.Scale);
					local traceOffset = Vector(trace.X * mo.FlipFactor, trace.Y):RadRotate(moAngle);
					
					wound.InheritedRotAngleOffset = traceOffset.AbsRadAngle - (mo.HFlipped and 0 or math.pi);
					mo:AddWound(wound, Vector(offset.X, offset.Y), false);
				end
				if self.penet ~= 0 then
					local traceIncrement = Vector(trace.X, trace.Y):SetMagnitude(skipPx);
					local parent = nil;
					for i = 0, mo.Diameter/(1 + skipPx) do
						hitPos = hitPos + traceIncrement;
						local noID = SceneMan:GetMOIDPixel(hitPos.X, hitPos.Y);
						if noID == rte.NoMOID or noID ~= moID then
							--Move one step down in attachable hierarchy
							if not parent and mo:GetParent() and noID == mo:GetParent().ID then
								parent = mo:GetParent();
								mo = parent;
								moID = parent.ID;
							else
								local woundName = mo:GetExitWoundPresetName();
								if woundName ~= "" then
									local wound = CreateAEmitter(woundName);
									wound.EmitCountLimit = wound.EmitCountLimit * diminishedMultiplier;
									wound.BurstDamage = wound.BurstDamage * self.damageMultiplier;
									wound.Scale = wound.Scale * math.sqrt(diminishedMultiplier);
									for em in wound.Emissions do
										em.ParticlesPerMinute = em.ParticlesPerMinute * self.damageMultiplier;
										em.BurstSize = em.BurstSize * self.damageMultiplier;
										em.MaxVelocity = em.MaxVelocity * diminishedMultiplier;
										em.MinVelocity = em.MinVelocity * diminishedMultiplier;
									end
									local dist = SceneMan:ShortestDistance(mo.Pos, hitPos - traceIncrement, SceneMan.SceneWrapsX);
									local offset = Vector(dist.X * mo.FlipFactor, dist.Y):RadRotate(moAngle):SetMagnitude(dist.Magnitude - (wound.Radius - 1) * wound.Scale);
									local traceOffset = Vector(trace.X * mo.FlipFactor, trace.Y):RadRotate(moAngle);
									
									wound.InheritedRotAngleOffset = traceOffset.AbsRadAngle - (mo.HFlipped and math.pi or 0);
									mo:AddWound(wound, Vector(offset.X, offset.Y), false);
									
									--Spawn new bullet that will take a life of its own
									local bullet = CreateMOPixel("Hitscan Bullet", "4zK.rte");
									bullet.Pos = hitPos;
									bullet.Vel = Vector(trace.X, trace.Y):SetMagnitude(250 * (1 - rayLength/self.range));
									bullet.Team = self.Team;
									bullet.PinStrength = math.max(self.penet, 1);
									bullet.WoundDamageMultiplier = self.damageMultiplier * 0.5;
									MovableMan:AddParticle(bullet);
								end
								break;
							end
						end
					end
				end
			else	--Hit terrain
				local particleCount = 5;
				local randAngle = math.random() * math.pi;
				for i = 1, particleCount do
					local glow = CreateMOPixel("Particle Very Heavy Digger", "Base.rte");
					glow.Pos = gapPos;
					glow.Vel = Vector(math.random(25, 50), 0):RadRotate(randAngle + math.pi * 2 * i/particleCount);
					MovableMan:AddParticle(glow);
				end
				local glow = CreateMOPixel("Disintegration Glow 10", "Techion.rte");
				glow.Pos = gapPos;
				MovableMan:AddParticle(glow);
					
				self.ricochetSound:Play(hitPos);
				self.shake = self.shake + 1;
			end
		end
		self.shake = self.shake + (math.rad(aiming and self.SharpShakeRange or self.ShakeRange)/(self.shake + 1));
	end
	
	self.shake = self.shake > 0 and math.max(self.shake * 0.99 - 0.01, 0) or 0;
	self.RateOfFire = self.origFireRate / (1 + self.shake);
end