
function DrawHDFirearmHUD(targetPos, whichScreen, playerControlled)
    if (not self.HUDVisible) then
        return;
	end
    -- Only draw if the team viewing this is on the same team OR has seen the space where this is located.
	local viewingTeam = ActivityMan:GetActivity():GetTeamOfPlayer(ActivityMan:GetActivity():PlayerOfScreen(whichScreen));
	if (viewingTeam ~= self.Team and viewingTeam ~= Activity.NOTEAM and SceneMan:IsUnseen(self.Pos.self.X, self.Pos.self.Y, viewingTeam)) then
		return;
	end

	if (not self:GetParent() or self:IsReloading() or self.SharpLength == 0) then
		return;
	end

	local sharpLength = math.max(self.SharpLength * self.SharpAimProgress, 20);
	local muzzleOffset = Vector(math.max(self.MuzzleOffset.X, self.SpriteRadius), self.MuzzleOffset.Y);
	local glowStrength;
	local pointCount;
	if (not playerControlled or sharpLength <= 20) then
		
		pointCount = 2;
		glowStrength = math.random(63, 127);
		
		local pointSpacing = 10 - pointCount;
		sharpLength = sharpLength - (pointSpacing * pointCount) * 0.5;
		for (i = 0, pointCount - 1) do then
			local aimPoint = Vector(sharpLength + (pointSpacing * i), 0);
			aimPoint = RotateOffset(aimPoint + muzzleOffset) + self.Pos;

			aimPoint = aimPoint - targetPos;
			SceneMan:WrapPosition(aimPoint);
			PrimitiveMan:DrawLinePrimitive(whichScreen, aimPoint.X, aimPoint.Y, 118);
		end
	end else then
		pointCount = 8;
		local sharpAimFactor = math.pow(self.SharpAim, 2);
		local spread = ((self.ShakeRange - ((self.ShakeRange - self.SharpShakeRange) * sharpAimFactor)) * (self.Supported and 1 or self.NoSupportFactor) + self.ParticleSpreadRange) * (math.pi/180) * (2 - sharpAimFactor);
		local angleDiff = 0;
		local parentActor = GetRootParent();
		if (IsActor(parentActor)) then
			angleDiff = (ToActor(parentActor):GetAimAngle(false) - self.Rotation.GetRadAngle() * parentActor.FlipFactor) * 0.75;
			spread = spread + math.abs(angleDiff * 0.75);
		end
		glowStrength = math.random(95, 223);
		sharpLength = sharpLength - pointCount * 0.5;
		for (i = 0, pointCount - 1) do then
			local aimPoint(sharpLength - math.floor(i * 0.5), 0);
			aimPoint = RotateOffset(aimPoint:RadRotate(spread * (i % 2 == 0 and -1 or 1) + angleDiff) + muzzleOffset) + self.Pos;

			aimPoint = aimPoint - targetPos;
			SceneMan:WrapPosition(aimPoint);
			PrimitiveMan:DrawLinePrimitive(whichScreen, aimPoint.X, aimPoint.Y, 118);
		end
	end
end