function Create(self)

	self.range = self.Vel.Magnitude * rte.PxTravelledPerFrame * (self.Lifetime/TimerMan.DeltaTimeMS);
	self.penet = self.PinStrength;
	
	self.parentID = rte.NoMOID;
	
	local checkPos = self.Pos - Vector(self.Vel.X, self.Vel.Y):SetMagnitude(-10);
	local moID = SceneMan:GetMOIDPixel(checkPos.X, checkPos.Y);
	if moID ~= rte.NoMOID then
		local mo = ToMOSRotating(MovableMan:GetMOFromID(moID));
		if mo.Team == self.Team then
			self.parentID = mo:GetRootParent().ID;
		end
	end

	self.ricochetSound = CreateSoundContainer("Ricochet", "Base.rte");
end

function Update(self)

	local startPos = self.Pos;
	local hitPos = Vector(startPos.X, startPos.Y);
	local gapPos = Vector(startPos.X, startPos.Y);
	local skipPx = 2;
	local trace = Vector(self.Vel.X, self.Vel.Y):SetMagnitude(self.range);
	--Use higher pixel skip to find a rough estimate
	local rayLength = SceneMan:CastObstacleRay(startPos, trace, hitPos, gapPos, self.parentID, self.Team, rte.grassID, skipPx * 2);

	local traceDist = SceneMan:ShortestDistance(startPos, hitPos, SceneMan.SceneWrapsX);
	PrimitiveMan:DrawLinePrimitive(startPos + traceDist * math.random(), startPos + traceDist * math.random(), 118);
	
	local lodged = true;
	
	if rayLength >= 0 then
	
		rayLength = SceneMan:CastObstacleRay(gapPos, Vector(trace.X, trace.Y):SetMagnitude(self.range - rayLength + 1), hitPos, gapPos, self.parentID, self.Team, rte.airID, skipPx);

		local moID = SceneMan:GetMOIDPixel(hitPos.X, hitPos.Y);
		if moID ~= rte.NoMOID and moID ~= self.parentID then
			local mo = ToMOSRotating(MovableMan:GetMOFromID(moID));
			mo:AddImpulseForce(Vector(trace.X, trace.Y):SetMagnitude(100 * self.WoundDamageMultiplier), Vector());
			local moAngle = -mo.RotAngle * mo.FlipFactor;
			local diminishedMultiplier = math.sqrt(self.WoundDamageMultiplier);
			local woundName = mo:GetEntryWoundPresetName();
			if woundName ~= "" then
				local wound = CreateAEmitter(woundName);
				wound.EmitCountLimit = wound.EmitCountLimit * diminishedMultiplier;
				wound.BurstDamage = wound.BurstDamage * self.WoundDamageMultiplier;
				wound.Scale = wound.Scale * math.sqrt(diminishedMultiplier);
				for em in wound.Emissions do
					em.ParticlesPerMinute = em.ParticlesPerMinute * self.WoundDamageMultiplier;
					em.BurstSize = em.BurstSize * self.WoundDamageMultiplier;
					em.MaxVelocity = em.MaxVelocity * diminishedMultiplier;
					em.MinVelocity = em.MinVelocity * diminishedMultiplier;
				end
				local dist = SceneMan:ShortestDistance(mo.Pos, hitPos, SceneMan.SceneWrapsX);
				local offset = Vector(dist.X * mo.FlipFactor, dist.Y):RadRotate(moAngle):SetMagnitude(dist.Magnitude - (wound.Radius - 1) * wound.Scale);
				local traceOffset = Vector(trace.X * mo.FlipFactor, trace.Y):RadRotate(moAngle);
				
				wound.InheritedRotAngleOffset = traceOffset.AbsRadAngle - (mo.HFlipped and 0 or math.pi);
				mo:AddWound(wound, Vector(offset.X, offset.Y), math.random() < 0.5);
			end
			if self.penet ~= 0 then
				local traceIncrement = Vector(trace.X, trace.Y):SetMagnitude(skipPx);
				local parent = nil;
				for i = 0, mo.Diameter/(1 + skipPx) do
					hitPos = hitPos + traceIncrement;
					noID = SceneMan:GetMOIDPixel(hitPos.X, hitPos.Y);
					if noID == rte.NoMOID or noID ~= moID then
						--Move one step down in attachable hierarchy
						if not parent and mo:GetParent() and noID == mo:GetParent().ID then
							parent = mo:GetParent();
							mo = parent;
							moID = parent.ID;
						else
							local woundName = mo:GetExitWoundPresetName();
							if woundName ~= "" then
								local wound = CreateAEmitter(woundName);
								wound.EmitCountLimit = wound.EmitCountLimit * diminishedMultiplier;
								wound.BurstDamage = wound.BurstDamage * self.WoundDamageMultiplier;
								wound.Scale = wound.Scale * math.sqrt(diminishedMultiplier);
								for em in wound.Emissions do
									em.ParticlesPerMinute = em.ParticlesPerMinute * self.WoundDamageMultiplier;
									em.BurstSize = em.BurstSize * self.WoundDamageMultiplier;
									em.MaxVelocity = em.MaxVelocity * diminishedMultiplier;
									em.MinVelocity = em.MinVelocity * diminishedMultiplier;
								end
								local dist = SceneMan:ShortestDistance(mo.Pos, hitPos - traceIncrement, SceneMan.SceneWrapsX);
								local offset = Vector(dist.X * mo.FlipFactor, dist.Y):RadRotate(moAngle):SetMagnitude(dist.Magnitude - (wound.Radius - 1) * wound.Scale);
								local traceOffset = Vector(trace.X * mo.FlipFactor, trace.Y):RadRotate(moAngle);
								
								wound.InheritedRotAngleOffset = traceOffset.AbsRadAngle - (mo.HFlipped and math.pi or 0);
								mo:AddWound(wound, Vector(offset.X, offset.Y), math.random() < 0.5);
								
								self.Pos = hitPos;
								self.range = self.range - rayLength;
								self.penet = self.penet - 1;
								lodged = false;
							end
							break;
						end
					end
				end
			end
		else	--Hit terrain
			local particleCount = 5;
			local randAngle = math.random() * math.pi;
			for i = 1, particleCount do
				local glow = CreateMOPixel("Ricochet Particle", "4zK.rte");
				glow.Pos = gapPos;
				glow.Vel = Vector(math.random(10, 40), 0):RadRotate(randAngle + math.pi * 2 * i/particleCount);
				MovableMan:AddParticle(glow);
			end
			local glow = CreateMOPixel("Glow Yellow Tiny", "4zK.rte");
			glow.Pos = gapPos;
			MovableMan:AddParticle(glow);
				
			self.ricochetSound:Play(hitPos);
			lodged = true;
		end
	end
	if lodged then
		self.ToDelete = true;
	end
end