--[[
SceneMan:CastStrengthRay
Traces along a Vector and shows where along that ray there is an encounter with a pixel of a material with strength more than or equal to a specific value.

Arguments:
- The starting position.
- The vector to trace along.
- The strength value of screen that can terminate the ray.
- A Vector reference which will be filled out with the absolute location of the found terrain pixel of more than or equal to the strength above.
- For every pixel checked along the line, how many to skip between them for optimization reasons. 0 = every pixel is checked.
- A material ID to ignore, IN ADDITION to Air.
- Whether the ray should wrap around the scene if it crosses a seam.

Return value:
- Whether a material of equal or more strength was found along the ray. If not, the fourth parameter has been set as the last position of the ray.
]]--
	local startPos = self.Pos;
	local trace = Vector(self.Radius, 0):RadRotate(self.RotAngle);
	
	local strengthThreshold = 10;
	local hitPos = Vector();
	
	local ignoreID = rte.grassID;
	local skipPx = 5;
	local wrapsX = SceneMan.SceneWrapsX;
	
	local rayHit = SceneMan:CastStrengthRay(startPos, trace, strengthThreshold, hitPos, skipPx, ignoreID, wrapsX);
	
	if rayHit then

		print("Terrain found at (" .. hitPos.X .. ", " .. hitPos.Y .. ")");
		
		PrimitiveMan:DrawCirclePrimitive(hitPos, skipPx, 13);
		
	end
	
	--How it looks without the arguments stored as local values:
	local rayHit = SceneMan:CastStrengthRay(self.Pos, Vector(self.Radius, 0):RadRotate(self.RotAngle), 10, Vector(), 5, rte.grassID, SceneMan.SceneWrapsX);